//
//  MINTLResponderInspectorPanelController.swift
//  BoxShadowExample
//
//  Created by narita on 2018/11/30.
//  Copyright © 2018 narita. All rights reserved.
//

import Cocoa

class MINTLResponderInspectorPanelController: NSWindowController
{
    // MARK: - class methods
    
    static let sharedInstance =
    {
        return MINTLResponderInspectorPanelController()
    }()

    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods

    override func windowDidLoad()
    {
        super.windowDidLoad()
        
        // ToDo: KVOの表記が煩雑すぎる。cocoa bindingの様にkeyPathで表記できないか検討すること
        // keyWindow.firstResponderとmainWindow.firstResponderの変化を捕まえてり表示をリフレッシュする
        
        self.keyWindowObservation = NSApp.observe( \.keyWindow, options: [.new])
        { _, change in
            
            // keyWindowのfirstResponderの変化を監視する
            if let theNewWindow = change.newValue as? NSWindow
            {
                self.keyWindowFirstResponderObservation = theNewWindow.observe( \.firstResponder )
                { _, _ in
                    self.refreshInspector(nil)
                }
            }
            else
            {
                self.keyWindowFirstResponderObservation = nil
            }
            
            self.refreshInspector(nil)
        }
        
        // 最初の一回め
        self.keyWindowFirstResponderObservation = NSApp.keyWindow?.observe( \.firstResponder )
        { _, _ in
            self.refreshInspector(nil)
        }
        
        self.mainWindowObservation = NSApp.observe( \.mainWindow, options: [.new])
        { _, change in
            
            // keyWindowのfirstResponderの変化を監視する
            if let theNewWindow = change.newValue as? NSWindow
            {
                self.mainWindowFirstResponderObservation = theNewWindow.observe( \.firstResponder )
                { _, _ in
                    self.refreshInspector(nil)
                }
            }
            else
            {
                self.mainWindowFirstResponderObservation = nil
            }
            
            self.refreshInspector(nil)
        }

        // 最初の一回め
        self.mainWindowFirstResponderObservation = NSApp.mainWindow?.observe( \.firstResponder )
        { _, _ in
            self.refreshInspector(nil)
        }
        
        
    }
    
    // MARK: - action methods
    
    @IBAction func refreshInspector(_ sender: Any?)
    {
        // 
        self.textView!.string = NSResponder.debugResponderChain()
    }
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @IBOutlet var textView : NSTextView?

    override var windowNibName: NSNib.Name
    {
        return NSNib.Name("MINTLResponderInspectorPanelController")
    }

    @objc var keyWindowObservation: NSKeyValueObservation? = nil
    @objc var keyWindowFirstResponderObservation: NSKeyValueObservation? = nil
    
    @objc var mainWindowObservation: NSKeyValueObservation? = nil
    @objc var mainWindowFirstResponderObservation: NSKeyValueObservation? = nil

    // MARK: - Utility methods
}
