//
//  MINTLBoxShadow.swift
//  BoxShadowExample
//
//  Created by narita on 2018/11/26.
//  Copyright © 2018 narita. All rights reserved.
//

import Cocoa

@IBDesignable

class MINTLBoxShadow: NSBox
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    override func draw(_ dirtyRect: NSRect)
    {
        super.draw(dirtyRect)
       
        NSGraphicsContext.stackState
        {
            // 前面色の設定
            NSColor.white.setFill()
            
            // 影の設定
            NSShadow(radius: self.mintSdwRadius, offset: self.mintSdwOffset ).set()
            
            // 描画する箱の大きさを設定
            let theBoxRect = self.bounds.insetBy(
                                            dx: self.contentViewMargins.width,
                                            dy: self.contentViewMargins.height )
            
            //  内側影だけ描画するために、クリッピングを作成
            NSBezierPath.boxClippingPath(  rect: theBoxRect,
                                          depth: self.mintSdwRadius ).setClip()

            // 内側の影の描画
            NSBezierPath.boxInsideShadowPath(  rect: theBoxRect,
                                              depth: self.mintSdwRadius,
                                              sides: self.sides ).fill()
            
            //  外側の影だけ描画するために、クリッピングを作成
            NSBezierPath.boxClippingPath(  rect: theBoxRect,
                                           depth: -self.mintSdwRadius ).setClip()
            // 外側の影の描画
            NSBezierPath.boxOutsideShadowPath(  rect: theBoxRect,
                                                depth: self.mintSdwRadius,
                                                sides: self.sides ).fill()
        }

    }

    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @IBInspectable var mintSdwOffset: NSSize = CGSize( w:  0, h: -1  )
    {
        didSet { self.setNeedsDisplay(self.bounds) }
    }
     
    @IBInspectable var mintSdwRadius: CGFloat = 8
    {
        didSet { self.setNeedsDisplay(self.bounds) }
    }

    var sides: MINTLBoxShadowSides = MINTLBoxShadowSides()
    {
        didSet { self.setNeedsDisplay(self.bounds) }
    }
    
    // -- top --
    @IBInspectable var top: NSInteger
    {
        get          { return self.sides.top.rawValue }
        set(newValue){ self.sides.top = MINTLBoxShadowSide(rawValue: newValue)! }
    }
    
    @objc class func keyPathsForValuesAffectingTop() -> Set<String>
    {
        return [ "sides" ]
    }

    // -- bottom --
    @IBInspectable var bottom: NSInteger
    {
        get          { return self.sides.bottom.rawValue }
        set(newValue){ self.sides.bottom = MINTLBoxShadowSide(rawValue: newValue)! }
    }
    
    @objc class func keyPathsForValuesAffectingBottom() -> Set<String>
    {
        return [ "sides" ]
    }

    
    // -- right --
    
    @IBInspectable var right: NSInteger
    {
        get          { return self.sides.right.rawValue }
        set(newValue){ self.sides.right = MINTLBoxShadowSide(rawValue: newValue)! }
    }
    
    @objc class func keyPathsForValuesAffectingRight() -> Set<String>
    {
        return [ "sides" ]
    }

    // -- left --

    @IBInspectable var left: NSInteger
    {
        get          { return self.sides.left.rawValue }
        set(newValue){ self.sides.left = MINTLBoxShadowSide(rawValue: newValue)! }
    }
    
    @objc class func keyPathsForValuesAffectingLeft() -> Set<String>
    {
        return [ "sides" ]
    }

    // MARK: - Utility methods
    
}
