//
//  NSBezierPath+MINTLBoxShadow.swift
//  BoxShadowExample
//
//  Created by narita on 2018/11/26.
//  Copyright © 2018 narita. All rights reserved.
//

import Foundation
import Cocoa

// bitMapに書き出して、layer化の時に使うつもり
extension NSImage
{
    public convenience init(size: NSSize, closure: () -> ())
    {
        self.init(size: size)
        
        self.lockFocus()
        closure()
        self.unlockFocus()
    }
    
    open func focus(closure: () -> ())
    {
        self.lockFocus()
        closure()
        self.unlockFocus()
    }
}

extension NSGraphicsContext
{
    // ローカルスコープ(クロージャーだけど)に囲い込むことで、元に戻し忘れをなくす
    public static func stackState(closure: () -> ())
    {
        NSGraphicsContext.saveGraphicsState()
        closure()
        NSGraphicsContext.restoreGraphicsState()
    }
}

extension CGSize
{
    // 省略表記
    public init(w: Int, h: Int)
    {
        self.init(width: w, height: h)
    }
    
    public init(w: Double, h: Double)
    {
        self.init(width: w, height: h)
    }
}

extension NSShadow
{
    // 省略表記
    public convenience init(radius: CGFloat, offset: CGSize = .zero, color: NSColor? = nil)
    {
        self.init()
        
        self.shadowBlurRadius = radius
        self.shadowOffset     = offset
        self.shadowColor      = color != nil ? color : self.shadowColor
    }
}

extension NSRect
{
    public var topLeft: NSPoint
    {
        return NSMakePoint( NSMinX(self), NSMaxY(self) )
    }

    public var topMiddle: NSPoint
    {
        return NSMakePoint( NSMidX(self), NSMaxY(self) )
    }

    public var topRight: NSPoint
    {
        return NSMakePoint( NSMaxX(self), NSMaxY(self) )
    }

    public var middleLeft: NSPoint
    {
        return NSMakePoint( NSMinX(self), NSMidY(self) )
    }
    
    public var middleMiddle: NSPoint
    {
        return NSMakePoint( NSMidX(self), NSMidY(self) )
    }
    
    public var middleRight: NSPoint
    {
        return NSMakePoint( NSMaxX(self), NSMidY(self) )
    }

    public var bottomLeft: NSPoint
    {
        return NSMakePoint( NSMinX(self), NSMinY(self) )
    }
    
    public var bottomMiddle: NSPoint
    {
        return NSMakePoint( NSMidX(self), NSMinY(self) )
    }
    
    public var bottomRight: NSPoint
    {
        return NSMakePoint( NSMaxX(self), NSMinY(self) )
    }
}
