//
//  NSResponder+MINTLResponderInspector.swift
//
//
//  Created by narita on 2017/09/06.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

extension NSObject
{
    public var debugResponderName: String
    {
        if let theSelf = self as? NSUserInterfaceItemIdentification
        {
            let theIDString = theSelf.identifier == nil
                ? String(format: "0x%016x", self) : theSelf.identifier!.rawValue
            
            return "\(self.className): \(theIDString)"
        }
        
        return "\(self.className): \(String(format: "0x%016x", self))"
    }
}

extension NSResponder
{
    static func debugResponderChain() -> String
    {
        var theResult: String = ""
        
        if NSApp.keyWindow != NSApp.mainWindow, let theKeyWindow = NSApp.keyWindow
        {
            theResult += "=== keyWindow ===\r"
            
            theResult += "\(NSResponder.inspectResponderChain(responder: theKeyWindow.firstResponder))\r"
        }

        if let theMainWindow = NSApp.mainWindow
        {
            theResult += "=== mainWindow ===\r"

            theResult += "\(NSResponder.inspectResponderChain(responder: theMainWindow.firstResponder))\r"
        }
        
        return theResult
    }
    
    static func inspectResponderChain(responder: NSResponder?) -> String
    {
        var theResult: String = ""
        var theResponder      = responder
        var theIndent: Int    = 1
        var theIndentString: String = ""
        var theAppClassFlag: Bool = false
        
        while( theResponder != nil )
        {
            theIndentString = String(repeating:"  ", count: theIndent)
            
            theResult += "\(theIndentString)[R]\(theResponder!.debugResponderName)\r"
            
            // delegateを持っている場合は、delegateも表示する
            if theResponder!.responds(to: NSSelectorFromString("delegate"))
            {
                if let theDelegate: NSObject = theResponder?.value(forKey: "delegate") as? NSObject
                {
                    theResult += "\(theIndentString)[D]\(theDelegate.debugResponderName)\r"
                }
            }
            
            if theResponder!.isKind(of: NSWindowController.self)
            {
                if let theDocument = theResponder!.value(forKey: "document") as? NSDocument
                {
                    theResult += "\(theIndentString)[O]\(theDocument.debugResponderName)\r"
                }
            }
            
            if theResponder == NSApp
            {
                theAppClassFlag = true
            }
            
            theResponder = theResponder?.nextResponder
            theIndent = theIndent + 1
        }
        
        theResult += "\r"

        if theAppClassFlag == false
        {
            theResponder = NSApp
            
            theResult += "\(theIndentString)[R]\(theResponder!.debugResponderName)\r"
            
            // delegateを持っている場合は、delegateも表示する
            if theResponder!.responds(to: NSSelectorFromString("delegate"))
            {
                if let theDelegate: NSObject = theResponder?.value(forKey: "delegate") as? NSObject
                {
                    theResult += "\(theIndentString)[D]\(theDelegate.debugResponderName)\r"
                }
            }
        }
        
        // ToDo: Document-base-appであるかの判定方法がわからないので、無条件で表示
        if true
        {
            theResult += "\(theIndentString)[O]\(NSDocumentController.shared.debugResponderName)\r"
        }

        
        return theResult
    }    
}
